#include <stdio.h>
#include <fcntl.h>

#define ERROR 0xFFFF

typedef struct { unsigned ax,bx,cx,dx,si,di,ds,es; } regfile;

typedef unsigned int paragraph;

extern int errno;

paragraph get_big_mem(size,avail)
	unsigned size;
	unsigned *avail;
{
	regfile regs;
	if (size % 16 != 0)	/* not an even paragraph amount */
		size += 15;
	size >>= 4;			/* shift to a paragraph count */
	regs.ax = 0x4800;	/* allocate memory call */
	regs.bx = (unsigned int)size;
	if (sysint(0x21,&regs,&regs) & 1)	/* error returned from sysint */
	{
		*avail = ((unsigned)regs.bx) << 4;
		return 0;
	}
	*avail = 0;
	return regs.ax;	/* paragraph of allocated block */
}

int free_big_mem(base)
	paragraph base;
{
	regfile regs;
	regs.ax = 0x4900;	/* free block */
	regs.es = base;		/* base paragraph */
	if (sysint(0x21,&regs,&regs) & 1)
		return regs.ax;
	return 0;
}

unsigned dread(handle,buf,count)
	unsigned handle; paragraph buf; unsigned count;
{
	regfile regs;
	regs.ax = 0x3f00;
	regs.bx = handle;
	regs.ds = buf;
	regs.dx = 0;
	regs.cx = count;
	if (sysint(0x21,&regs,&regs) & 1)
	{
		errno = regs.ax;
		return ERROR;
	}
	return (int)regs.ax;
}

unsigned dwrite(handle,buf,count)
	unsigned handle; paragraph buf; unsigned count;
{
	regfile regs;
	regs.ax = 0x4000;
	regs.bx = handle;
	regs.ds = buf;
	regs.dx = 0;
	regs.cx = count;
	if (sysint(0x21,&regs,&regs) & 1)
	{
		errno = regs.ax;
		return ERROR;
	}
	return (int)regs.ax;
}
#ifdef NOTDEF
#define O_RDONLY	0
#define O_WRONLY	1 
#define O_RDWR		2
#define O_CREAT		3

int dopen(name,mode)
	char *name;
	unsigned mode;
{
	extern unsigned _dsval;
	regfile regs;
	if (mode > O_CREAT)
		return ERROR;
	if (mode != O_CREAT)
		regs.ax = 0x3D00 | mode;
	else
	{
		regs.ax = 0x3c00;
		regs.cx = 0;
	}
	regs.ds = _dsval;
	regs.dx = (unsigned)name;
	if (sysint(0x21,&regs,&regs) & 1)
	{
		errno = regs.ax;
		return ERROR;
	}
	return (int)regs.ax;
}

int dclose(handle)
	int handle;
{
	regfile regs;
	regs.ax = 0x3E00;
	regs.bx = handle;
	if (sysint(0x21,&regs,&regs) & 1)
	{
		errno = regs.ax;
		return ERROR;
	}
	return 0;
}
#endif
#define BUFSIZE ((long)((long)63*1024))
main(argc,argv)
	char **argv;
{
	paragraph buffer; unsigned avail,junk;
	unsigned actual_read,in,out;
	if (argc != 3)
	{
		fprintf(stderr,"bigcp : usage bigcp from to\n");
		exit(ERROR);
	}
	if (0 == (buffer = get_big_mem(BUFSIZE,&avail)))
	{
		if (0 == (buffer = get_big_mem(avail,&junk)))
		{
			fprintf(stderr,"no memory available\n");
			exit(ERROR);
		}
	}
	else 
		avail = BUFSIZE;

	if (-1 == (in = open(argv[1],O_RDONLY)))
	{
		perror(argv[1]);
		exit(ERROR);
	}

	if (-1 == (out = creat(argv[2],0666)))
	{
		perror(argv[2]);
		exit(ERROR);
	}

	/* do the file copy */
	while (0 != (actual_read = dread(in,buffer,avail)) && actual_read != ERROR)
	{
		if (ERROR == dwrite(out,buffer,actual_read))
		{
			fprintf(stderr,"error writing %s\n",argv[2]);
			perror("");
			exit(ERROR);
		}
	}
	if (ERROR == actual_read)
	{
		fprintf(stderr,"error reading %s\n",argv[1]);
		perror("");
		exit(ERROR);
	}
	close(in); close(out);
}
