/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.008        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: main.c
 * Program name: pc-shell
 * Source of file: The Public Domain Software Library.
 * Purpose: A UNIX type shell environment for MS-DOS
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

/*
** $Log:   C:/shell/vcs/main.c_v  $
**
**   Rev 3.6   02 Oct 1986 13:11:24   kent
**lint pass over everything
**
**   Rev 3.5   01 Oct 1986 13:27:24   kent
**Put all globals into shell.h
**
**   Rev 3.4   01 Oct 1986  9:22:48   kent
**Initial revision.
*/
/*lint -e527 -e10 -e720 -e529 -e715 */
#define MAIN
#include "shell.h"
#ifdef MSC
/* setup_crit() {} restore_crit() {} */
_setenvp() {}
#endif
jmp_buf critical;	/* used by critical error handler */

char get_switch()
{
	struct registers regs;
	regs.ax = 0x3700;
	(void)sysint21(&regs,&regs);
	return (char)(regs.dx & 0xFF);
}

void set_switch(c)
char c;
{
	struct registers regs;
	regs.ax = 0x3701;
	regs.dx = (int)c;
	(void)sysint21(&regs,&regs);
}

static jmp_buf env;

void main(argc,argv)
	int argc; char **argv;
{
	int files[3];
	void exit();
	int i = 1;
	char argbuf[64];
	char *home,*buf, *savestr();
	signal(SIGINT,SIG_IGN);	/* ignore breaks */
	argv[0] = "shell";		/* put our name in */
	/* initialize local environment */
	init_env();
	/* put the command line arguments into the environment */
	while (--argc)
	{
		++argv;
		if ((*argv)[0] == '-')
		{
			char *opt = &(*argv)[1];
			while (*opt)
			{
				switch(toupper(*opt))
				{
				case 'V':
					verbose = 1;
					break;
				case 'S':	/* execute startup command file */
					buf = savestr("shell.rc");
					if (!filep(buf))
						if (NULL != (home = get_env("HOME")))
						{
							Free(buf);
							buf = MustMalloc((unsigned)strlen(home)+11);
							sprintf(buf,"%s/shell.rc",home);
						}
					if (filep(buf))
					{
						(void)std_save(files);
						close(0);
						(void)open(buf,O_RDONLY);
						quiet = 1;
						(void)cli();
						close(0);
						std_restore(files);
					}
					Free(buf);
					break;
				case 'B':
					/* don't map backslashes to forward slashes */
					no_map_backslashes++;
					env_change_backslashes();
					break;
				default:
					goto add_to_env;
					break;
				}
				opt++;
			}
			continue;
		}
add_to_env:
		sprintf(argbuf,"%d=%s",i,*argv);
		(void)add_env(argbuf);
		++i;
	}
	/* change switch character */
	oldswitch = get_switch();
	if (!no_map_backslashes)
		set_switch('-');
	setup_crit();
	/* set up temporary path */
	{
		char *tmp; 
		if (NULL != (tmp = get_env("TEMP")))
		{
			pipename[0] = MustMalloc(65);
			pipename[1] = MustMalloc(65);
			herename = MustMalloc(65);
			sprintf(pipename[0],"%s001",tmp);
			sprintf(pipename[1],"%s002",tmp);
			sprintf(herename,"%s003",tmp);
		}
		else
		{
			/* relative to current directory */
			pipename[0] = "001";
			pipename[1] = "002";
			herename = "003";
		}
	}
	(void)cli();
	restore_crit();
	if (!no_map_backslashes)
		set_switch(oldswitch);
	exit(0);
}

static jmp_buf shenv;
jmp_buf *exitenv;

void shintr()
{
	signal(SIGINT,SIG_IGN);
	longjmp(shenv,-1);
}

int sh(argc,argv)
	int argc; char **argv;
{
	int i = 1;
	int savequiet,saveverbose = verbose;
	char *argbuf;
	void (*oldsig)();

	oldsig = signal(SIGINT,SIG_IGN);
	argbuf= MustMalloc((unsigned)CMDBUFSIZ);
	if (-1 == setjmp(shenv))
	{
		fprintf(stderr,"interrupted\n");
		free(argbuf);
		signal(SIGINT,oldsig);
		return 1;
	}
	signal(SIGINT,shintr);
	/* put the command line arguments into the environment */
	while (--argc)
	{
		++argv;
		if ((*argv)[0] == '-')
		{
			char *opt = &(*argv)[1];
			while (*opt)
			{
				switch(toupper(*opt))
				{
				case 'V':
					verbose = 1;
					break;
				default:
					fprintf(stderr,"unknown option %c\n",*opt);
					break;
				}
				opt++;
			}
			continue;
		}
		sprintf(argbuf,"%d=",i);
		argbuf = addtoline(argbuf,*argv,0);
		(void)add_env(argbuf);
		++i;
	}
	Free(argbuf);
	savequiet = quiet;
	(void)cli();
	quiet = savequiet;
	verbose = saveverbose;
	return 0;
}

int _getc(fd)
	int fd;
{
#ifdef AZTEC86
	extern unsigned _dsval;
#endif
	char c;
	struct registers regs;
	int result;
	regs.ax = 0x3f00;	/* read */
	regs.bx = fd;
	regs.cx = 1;
	regs.dx = (unsigned)&c;
#ifdef AZTEC86
	regs.ds = _dsval;
	if (sysint21(&regs,&regs)&1 || regs.ax == 0)
#endif
#ifdef MSC
	GETSEGS(&regs);
	sysint21(&regs,&regs);
	if (regs.cflag || regs.ax == 0)
#endif
		return EOF;
	if (c == '\r')	/* if we see a carriage return, a newline must follow */
		return _getc(fd);
	return c;
}

int getline(fd,cmdbuf)
char *cmdbuf;
{
	register int c;
	while (EOF != (c = _getc(fd)))
	{
		switch(c)
		{
		case '\n':
			*cmdbuf = '\0';
			return 1;
		case '\\':
			switch (c=_getc(fd))
			{
			case EOF:
				return 0;
			case '\n':
				if (!quiet)
					write(2,"> ",2);
				continue;
			default:
				*cmdbuf++ = '\\';
			}
			/* fall into default case */
		default:
			*cmdbuf++ = c;
		}
	}
	if (c==EOF)
		return 0;
	return 1;
}


int cli()
{
		hist_env *save;
		char *cmdbuf,*savestr();
		int parseline();

		jmp_buf myenv,*saveenv;			/* my stack environment */
		quiet = !isatty(0);		/* quiet = batch shell */
		save = _current;
		_current = MustMalloc((unsigned)sizeof(hist_env));
		if (!quiet)
			_current->history = (char **)MustMalloc((unsigned)10*sizeof(char *));
		cmdbuf= MustMalloc((unsigned)CMDBUFSIZ);
		_current->histsize = 10;
		_current->j = 0;
		if (-1 == setjmp(myenv))
			goto subshelldone;
		saveenv = exitenv;	/* the old jmp buf pointer */
		exitenv = (jmp_buf *)myenv;	/* who gets jumped to on exit */
		for (;;)
		{
			/* kill tmp files */
			if (pdirty[0])
				pdirty[0] = 0,unlink(pipename[0]); 
			if (pdirty[1])
				pdirty[1] = 0,unlink(pipename[1]);
			/* hiscount is _current position in history */
			_current->hiscount = _current->j % _current->histsize; 
			histecho = 0;
			do
			{
				do
				{
					if(!quiet)
						do_prompt(_current);
					/* get user input */
					if (0 == getline(0,cmdbuf))
					{
subshelldone:
						if (!quiet)
						{
							int i;
							for (i = 0; i < _current->histsize;i++)
								if (_current->history[i])
									Free(_current->history[i]);
							Free(_current->history);
						}
						Free((char *)_current);
						Free(cmdbuf);
						_current = save;
						exitenv = saveenv;
						return 0;
					}
				} while (strlen(cmdbuf) == 0);
			/* parse command for compound statements and pipes */
			}
			while (-1 == parseline(cmdbuf))
				;
			_current->j++;
		}
		return 0;
}

void collapse(buf)
	register char *buf;
{
	for(;*buf;buf++)
		*buf=buf[1];
}

static char inpipe, pipenum;

char *do_pipe(startline)
	char *startline;
{
	char *execstring;
	if (inpipe)
	{
		inpipe = 0;
		execstring = savestr(startline);
		execstring = addtoline(execstring,"<",1);
		execstring = addtoline(execstring,pipename[pipenum],1);
		pdirty[pipenum]++;
		pipenum ^= 1;	/* flop pipe number */
		return execstring;
	}
	return startline;
}

int parseline(buf)
	char *buf;
{
	extern char *hexpand(),*vexpand(),*fexpand(),*aexpand();
	char *a,*h,*v,*f,*current,*startline,*file_expand;
	char *execstring; int command();
	inpipe = pipenum = 0;
#ifdef DEBUG
	fprintf(stderr,"buf = %s\n",buf);
#endif
	if (buf[0] == '#')
		return -1;
	if (!quiet)
	{
		/* Free previous history string */
		if (_current->history[_current->hiscount])
			Free(_current->history[_current->hiscount]);
		/* save _current in history array */
		_current->history[_current->hiscount] = savestr(buf);
	}
#ifdef DEBUG
	fprintf(stderr,"aexpand = %s\n",a);
#endif
	if (NULL == (a = aexpand(buf)))
		return -1;
	if (NULL == (h = hexpand(a)))	/* do history expansion */
	{
		free(a);
		return -1;
	}
	if (strcmp(h,a) != 0)	/* if we did any history expansion */
		++histecho;			/* echo history expansion */
	free(a);
#ifdef DEBUG
	fprintf(stderr,"hexpand = %s\n",h);
#endif
	if (NULL == (v = vexpand(h)))	/* do variable expansion */
	{
		free(h);
		return -1;
	}
	free(h);
#ifdef DEBUG
	fprintf(stderr,"vexpand = %s\n",v);
#endif
	if (0 == strcmp("ls",ntharg(v,UZERO)) ||
		0 == strcmp("dir",ntharg(v,UZERO)) ||
		(NULL != (file_expand=get_env("NOGLOB")) && 
			0 == strcmp(file_expand,"1"))
		)
	{
		f = v;	/* skip file name expansion */
	}
	else
	{
		if (NULL == (f = fexpand(v)))	/* do file name expansion */
		{
			free(v);	/* free history buffer */
			return -1;
		}
		free(v);
	}
#ifdef DEBUG
	fprintf(stderr,"fexpand = %s\n",f);
#endif
	/* f is now the fully expanded buffer */
	if (!quiet)
	{
		/* get rid of original copy used for history purposes */
		Free(_current->history[_current->hiscount]);
		/* save _current in history array */
		_current->history[_current->hiscount] = savestr(f);
#ifdef DEBUG
	fprintf(stderr,"_current->history[%d] = %s\n",
		_current->hiscount,_current->history[_current->hiscount]);
#endif
	}

	/* handle compound commands and conditionals */
	for(startline = current = f;*current;current++)
	{
		switch (*current)
		{
		case '\\':
			collapse(current);	/* blow away backslash */
			break;
		case '\'':	/* quoting mechanism */
		case '"':
		{
			char quotechar = *current;
			++current;
			while (*current && *current != quotechar)
			{
				if (*current == '\\')
					collapse(current);
				++current;
			}
			if ('\0' == *current)
			{
				fprintf(stderr,"No closing quotes on command line!\n");
				goto done;
			}
		}
			break;
		case ';':    
			if (current == startline)
				goto done;
			*current = '\0';
			execstring = do_pipe(startline);
			(void)command(execstring);
			if (execstring != startline)
				Free(execstring);
			startline = current+1;
			break;
		case '&':
			if (current == startline)
				goto done;
			if (current[1]== '&')
			{
				*current = '\0';
				++current;
				execstring = do_pipe(startline);
				(void)command(execstring);
				if (execstring != startline)
					Free(execstring);
				if (result != 0)
					goto done;
				startline = current + 1;
			}
			break;
		case '|':
			if (current == startline)
				goto done;
			if (current[1]== '|')
			{
				*current = '\0';
				++current;
				execstring = do_pipe(startline);
				(void)command(execstring);
				if (execstring != startline)
					Free(execstring);
				if (result == 0)
					goto done;
				startline = current + 1;
			}
			else
			{
				*current = '\0';
				execstring = savestr(startline);
				if (inpipe)
				{
					execstring = addtoline(execstring,"<",1);
					execstring = addtoline(execstring,pipename[pipenum],1);
					pdirty[pipenum]++;
					pipenum ^= 1;	/* flop pipe number */
				}
				execstring = addtoline(execstring,">",1);
				execstring = addtoline(execstring,pipename[pipenum],1);
				pdirty[pipenum]++;
				inpipe++;
				(void)command(execstring);
				Free(execstring);
				startline = current+1;
			}
		}
	}
	execstring = startline;
	if (inpipe)
	{
		execstring = addtoline(execstring,"<",1);
		execstring = addtoline(execstring,pipename[pipenum],1);
		pdirty[pipenum]++;
		pipenum ^= 1;	/* flop pipe number */
	}
	inpipe = 0;
	(void)command(execstring);
	if (execstring != startline)
		Free(execstring);
done:
	free(f);		/* get rid of buffer */
	return 0;
}

void onintr()
{
	longjmp(env,-1);
}

char *findonpath(name)
char *name;
{
	char *cp;
	static char path[64];
	char *xp;
	if (filep(name))
	{
		strcpy(path,name);
		return path;
	}
	if ((cp = get_env("PATH")) != 0) 
	{
		while (*cp) 
		{
			xp = path;
			while (*cp) 
			{
				if (*cp == ';') 
				{
					++cp;
					break;
				}
				*xp++ = *cp++;
			}
			*xp = '\0';
			if (path[0] != 0)
			{
				strcat(path,"/");
				strcat(path,name);
				if (filep(path))
					return path;
			}
		}
	}
	return NULL;
}

int command(current)
	register char *current;
{
	int files[3];		/* save current standard input and output */
	char resbuf[14];
	int returnval;
	char *scriptbuf,*buf,*redirect();
	void redirect_cleanup();
	extern int do_prog();
	register int i;
	int findcmd();
	char *quotestripper = NULL;
	char *nameonpath;
	char *a;	/* alias expansion */
#ifdef DEBUG
	fprintf(stderr,"current = %s\n",current);
	fprintf(stderr,"ntharg(current,0) = %s, get_env() = %s\n",
		ntharg(current,0),get_env(ntharg(current,0)));
#endif

	/* if there aren't any arguments, don't do anything */
	if (NULL == ntharg(current,UZERO))
	{
		returnval = -1;
		goto make_result;
	}
	
	/* do alias expansion */
	if (NULL == (a = aexpand(current)))	/* do alias expansion */
		return -1;

	/* set up the script name */
	scriptbuf = MustMalloc((unsigned)strlen(ntharg(a,UZERO))+4);
	strcpy(scriptbuf,ntharg(a,UZERO));
	strcat(scriptbuf,".sh");

	/* echo if necessary */
	if (isatty(2) && (verbose || (!quiet && histecho)))
		fprintf(stderr,"%s\n",a);

	/* save standard files */
	(void)std_save(files);

	/* do redirection */
	buf = redirect(a);
	free(a);	/* get rid of alias expanded version */

	if (buf == NULL)
		returnval = -1;
	else
	/* do internal commands */
	if (-1 != (i = findcmd(ntharg(buf,UZERO))))
	{
		if (-1 != setjmp(env))
		{
			char oldswitch;
			signal(SIGINT,onintr);
			if (-1 != setjmp(critical))
				result = _Croot(buf,commands[i].func);
		}
		signal(SIGINT,SIG_IGN);
	}
	/* do scripts */
	else if (NULL != (nameonpath = findonpath(scriptbuf)))
	{
		char *buf2,*buf3;
		if (-1 != setjmp(env))
		{
			buf2 = savestr(buf);
			buf2 = addtoline(buf2,"<",1);
			buf2 = addtoline(buf2,nameonpath,1);
			if (NULL != (buf3 = redirect(buf2)))
			{
				signal(SIGINT,onintr);
				if (-1 != setjmp(critical))
					returnval = result = _Croot(buf3,sh);
				Free(buf3);
			}
			else
				returnval = -1;
		}
		signal(SIGINT,SIG_IGN);
		Free(buf2);
	}
	/* try to execute the bugger as an external program */
	else
	{
		returnval = 0;
		for (i = 0;;i++)
		{
			char *cur_arg;
			char *quote_char;
			if ((cur_arg = ntharg(buf,(unsigned)i)) == NULL)
				break;
			else if (cur_arg == BADQUOTES)
			{
				fprintf(stderr,"No closing quotes on command line!");
				returnval = -1;
				break;
			}
			if (strchr("\"'",*cur_arg) != NULL)
			{
				++cur_arg;
				if (NULL != (quote_char = strrchr(cur_arg,cur_arg[-1])))
				{
					*quote_char = '\0';
					quotestripper = addtoline(quotestripper,cur_arg,1);
				}
				else
				{
					fprintf(stderr,"Missing closing quotes\n");
					if (quotestripper != NULL)
						free(quotestripper);
					quotestripper = NULL;
					break;
				}
			}
			else
			{
				quotestripper = addtoline(quotestripper,cur_arg,1);
			}
		}
		if (quotestripper)
		{
			char *dos;
			if (-1 != setjmp(critical))
				/* try to fork the program */
				if (666 == (result = forkp(quotestripper)))
					/* if that doesn't work, give command.com a whack at it */
					if (
						/* if NODOS=1, don't let DOS at it */
						(NULL != (dos=get_env("NODOS")) && 
							0 == strcmp(dos,"1"))
							|| system(quotestripper) == -1)
					{
						perror(current);
						returnval= -1;
					}
			free(quotestripper);
		}
		else
			returnval = -1;
	}
	if (buf)
		Free(buf);
	redirect_cleanup();
	std_restore(files);
	Free(scriptbuf);
make_result:
	sprintf(resbuf,"status=%d",result);
	(void)add_env(resbuf);
	return returnval;
}



void *Must_Malloc(
#ifdef MEMDEBUG
func,line,
#endif
size)
	register unsigned size;
#ifdef MEMDEBUG
char *func;int *line;
#endif
{
#ifdef MEMDEBUG
	extern int indent;
	register int i;
#endif
	register void *foo;

	if (NULL == (foo =malloc(size)))
	{
		perror("malloc");
		(void)my_exit(-1);
	}
	memset(foo,0,size);	/* zero buffer */
#ifdef MEMDEBUG
	for (i = 0; i < indent; i++)
		putc(' ',stderr);
	fprintf(stderr,"m %s %d: %x size %d\n",func,line,foo,size);
	indent++;
#endif
	return foo;
}
void *Must_Realloc(
#ifdef MEMDEBUG
func,line,
#endif
ptr,size)
	register unsigned size;
	void *ptr;
#ifdef MEMDEBUG
char *func;int *line;
#endif
{
	register void *foo;

	if (NULL == (foo =realloc(ptr,size)))
	{
		perror("realloc");
		(void)my_exit(-1);
	}
#ifdef MEMDEBUG
	fprintf(stderr,"r %s %d: %x size %d\n",func,line,foo,size);
#endif
	return foo;
}
/*lint -restore */
