.de CM
.sp 1
.in -4
.nf
.bo
$0
.fi
.in +4
.sp 1
.en
.TH PC-SHELL 1 "PC-DOS C Shell"
.SH NAME
pc-shell - pc command processor
.SH SYNOPSIS
.bo
pc-shell 
[-s] [-v] [-b] [arg1 ... argn]
.SH DESCRIPTION
pc-shell is a command processor for IBM-PC's and compatibles that emulates
some of the more desirable functions of the Berkeley UNIX* C-shell.  In
addition,  it implements PC-DOS versions of some of the common UNIX* commands 
-ls, mv, cp, etc.
.sp 1
The
.bo
-v
option (verbose) causes the program to echo all commands to the standard
error stream before executing them.
.sp 1
The
.bo
-b
option suppresses mapping of the backslash character to the forward
slash character in all operations that communicate with [PC|MS]DOS.
This is here primarily for compatibility with PC-NET, which seems to
choke on paths containing forward slashes.
.sp 1
.bo
arg1 ... argn
(the command line arguments) are put into the shell's environment as the
variables $1 through $n.
.sp 1
.SH "Wild Card Substitution" 
Ambiguous file names are expanded to a list of matching file names
on the command line.
This can be defeated by quoting, and by setting the NOGLOB
environment variable
to '1'.
.sp 1
Command lines passed to external programs are truncated to 128 characters.
.sp 1
.SH "History Substitution"
.sp 1
History substitution is a powerful means to save retyping of long
command lines.  It allows you to do things like re-execute the last
command, 
re-execute the last
command but redirect output to a file, or execute a new command with arguments
from previous command lines.  The last 20 commands are saved, and can be 
reviewed by typing the 'history' command.
.sp 1
Previous commands can be referred to by their number, or relative to
the 
current
command's number.  Parameters from previous commands can be seperated out and
used individually.
.sp 1
History substitutions specifications come in two parts - the command number 
specifier and the argument specifier, seperated by a colon.  The argument 
specifier is optional; if it is omitted, the entire command line is specified.
.sp 1
.nf
<command specifier> ::= !! | !n | !-n | <shortcut> 
!!  = last command
!n  = nth command
!-n = command n commands before current command number
!#  = the current command line
.sp 1
<arg specifier> ::= :[^$*] | [^$*] | :n | :n* | <searchstr> | <empty>
n   = number of argument (0 being the command name)
^   = first argument (i.e. argv[1])
$   = last argument
*   = ^-$, or nothing if only one word on command line
n*  = arguments n through $
.sp 1
<searchstr> ::= (initial characters of a previous command)
.sp 1
<history subst specification> ::= <command specifier><arg specifier>
.sp 1
.fi
This is not as complicatated as it may appear.  Here is an example session.
.sp 1
EXAMPLE
.sp 1
.nf
0% ls *.c
*.c
foo.c bar.c
1% more foo.c
/* edit the last argument of the last command */
2% edit !!:$			
/* go off and edit */
/* reference last argument of last command */
3% fgrep foo !!:$ bar.c 
FOO.C : foo
BAR.C : foo
/* edit the second thru the last args of command 3 */
4% edit !3:2*			
(go off and edit)
/* repeat last command */
%5 !!
(go off and edit)
/* remove the 1st argument of the command 2 before the current one */
%6 rm !-6:^
.sp 1
.fi
.sp 1
Several shortcut expressions (that don't fit into the formal
description above) are also allowed.
.bo
!$, !^, !*
are allowable synonyms for
.bo
!!:$, !!:^,
and
.bo
!!:*.
It is also possible to select a previous command with one or more
characters from the beginning of that command line.
.sp 1
.nf
.in +5
%1 edit foo.c	# edit a source file
%2 cc foo.c		# try and compile it
%3 !e			# repeat command #1
.br
.sp 1
.in -5
.fi
History substitution here is a compatible subset of the [U|XE]NIX C shell 
history substitution facility.  Cshell allows even weirder combinations.
.sp 1
.SH "Variable Substitution"
.sp 1
Shell variables are synonymous for our purposes with environment strings, i.e.
they are defined with the 'set' command.
.sp 1
Variables are referenced on the command line by prefacing a variable name by
a dollar sign.  Two dollar signs in a row signify a dollar sign character.
.sp 1
As mentioned above, command line arguments are contained in the shell variables,
$1 through $n, where
.bo
n
is the number of the last argument.
.nf
.sp 1
EXAMPLE
.sp 1
%0 set home = c:/
%1 echo $home
C:/
%2 ls $home
C:/*.*
command.com
%3 echo $path
C:/bin
.sp 1
.fi
And so on.
.br
.sp 1
.SH "Special variables"
There are a some shell variables that have special meanings for
the shell.  They can be given values with the
.bo
set
command.  They are
.br
.in +4
.SH NOCLOBBER
If
.bo
NOCLOBBER
is equal to '1', then existing files may not be destroyed by output redirection,
and output files to be appended to by
.bo
>>
must exist.
.sp 1
.SH PROMPT
The
.bo
PROMPT
environment string is handled the same way as it is by
.bo
COMMAND.COM.
There is one pc-shell-specific prompt string character
.ul
!
that specifies the current command index.  If no PROMPT environment string is
defined, then the default is
.bo
'$!% '.
.br
.in -4
.SH NOGLOB
The
.bo
NOGLOB
environment string, if set to '1', will suppress file name expansion.
.sp 1
.SH NODOS
The
.bo
NODOS 
environment string, if set to '1', will keep the shell from
invoking command.com to handle commands it doesn't understand.
.sp 1
.SH TEMP
The
.bo
TEMP
environment, gives the directory in which to place pipe files.  This
can be a ram disk, or just someplace out of harm's way.  This
variable must contain a trailing slash, if it specifies a subdirectory.
.sp 1
.SH "Multiple commands on one command line"
Command lines are split at semicolons. This can be defeated by quoting or
escaping.
.sp 1
EXAMPLE
.sp 1
.nf
%0 ls -l *.c ; make shell.exe ; exit
.fi
.ne 6
.SH "Conditional command execution"
If two commands are seperated by '&&', then the second will be executed only
if the first returns 0 as an exit code.
If two commands are seperated by '||', then the second will be executed only
the first command returns non-zero as an exit code.
.sp 1
Example
.sp 1
.in +4
.nf
make shell.exe && chmod +w /bin/shell.exe && mv shell.exe /bin 
.sp 1
.fi
If the make operation fails, then the chmod and the mv will not be executed.
.nf
.sp 1
make shell.exe || echo You blew it bub!
.sp 1
.fi
If the make operation fails, then the echo operation will be executed.
.sp 1
.in -4
.SH "Character Escapes, Shell Comments, and Argument Quoting"
Any character preceded by a 
.bo
\
(backslash) is copied unmodified to the command buffer.  This allows you
to suppress the special meanings of shell command characters, such as
.bo
'|', ';',
and
.bo
 '* '.
.br
.sp 1
.bo
#
is the shell comment character.  Anything on a line after a
.bo
#
character is ignored.
.sp 1
Command line arguments contained in quotes (single or double) may contain
blank space (i.e. blanks or tabs).  Variable substitution will take place
within strings surrounded by double quotes.  No interpretation takes place
within single quotes.  
.sp 1
Within double quotes, the 'C' language escape sequences \r,\n,\b,\f, and \a
are honored - i.e. they are mapped to their corresponding control characters.
.sp 1
.SH "Startup and Script Files"
.sp 1
If '-s' is specified on the command line the program will look for a file
called SHELL.RC in the current directory, and execute it before passing 
control to the console.  This allows you to set up all your alias commands.
It isn't a good idea to put an 'exit' command in your SHELL.RC file, as the
shell will terminate.
.sp 1
Any file whose extension is
.bo
.cc $
.sh
is run as a command file.  The environment string PATH is used to
locate the script file if it isn't in the current directory.
.sp 1
There is also a command
$bo
sh,
into which shell scripts whose extension is not
.sh
can be redirected as standard input.
$cc .
.nf
.sp 1
EXAMPLE
.sp 1
%0 sh <batch.fil		# use the sh command
%1 shell <batch.fil     # run the external program
.sp 1
.fi
Files ending in
.cc $
$bo
.bat
$cc .
files are passed to COMMAND.COM for execution.
.sp 1
.SH "INPUT/OUTPUT"
I/O redirection operates as it does under
.bo
COMMAND.COM
with some additional options:
.sp 1
.ti -4
.bo
<name
.sp 1
Opens the file
.bo
name
as the standard input.
.sp 1
.ti -4
.bo
<<word
.sp 1
reads the shell input up to a line which is identical to
.bo
word.
The resulting text is put into an anonymous temporary file, which is
given to the command as standard input.
.sp 1
.bo 4
.nf
>name
>!name
>&name
>&!name
.br
.in +4
.fi
.sp 1
The file
.bo
name 
is used as standard output.  If it doesn't exist, it's created; if it
exists, it is truncated, and its previous contents are lost.
.sp 1
If the variable
.bo
NOCLOBBER
is set, the file must not already exist, or an error results.  The forms
using an exclamation point override the NOCLOBBER variable's action.
.sp 1
The form
.bo
>&name
routes standard error along with standard output to
.bo
name.
.sp 1
.in -4
.bo 4
.nf
>>name
>>!name
>>&name
>>&!name
.br
.in +4
.fi
.sp 1
Uses the file
.bo
name
as standard output, like
.bo
>,
but places output at the end of file.  If the variable
.bo
NOCLOBBER
is set, it is an error if the file doesn't already exist.  The forms using an
exclamation point override the NOCLOBBER variable's action.
.sp 1
.SH "BUILT-IN COMMANDS"
.sp 1
Some of the internal commands are UNIX* style replacements for COMMAND.COM
internal commands, and some are included for convenience.
.sp 1
.nf
.in +4
Output of the 'commands' command
.sp 1
a:              alias           b:              c:
cat             cd              chdir           chmod
cls             commands        copy            cp
d:              del             dir             dump
e:              echo            era             erase
error           exit            f:              fgrep
g:              h:              hd              history
i:              j:              ls              md
mkdir           mon             more            mv
popd            pushd           pwd             rd
read            rm              rmdir           set
sh              switchar        tee             touch
unalias         version         y
.in -4
.fi
.br
.sp 1
There are many that are simply aliases, e.g. 'copy' and 'cp' invoke
the same program.
.br
.sp 1
.SH "COMMAND DESCRIPTION SYNTAX"
.nf
terms used in syntax explanations :

fname ::= PC-DOS ambiguous or unambiguous file or directory name.

uname ::= unambiguous PC-DOS file or directory name

string ::= any string of printable characters of arbitrary(<512) length.

filelist ::= filename [filename .. filename]

noargs ::= no arguments at all

space ::= any white space characters

[arg] ::= term is optional

envstring ::=	<string>=<string> | <string><space>=<space><string> |
                <string><space><string>
.fi
.br
.sp 1
.SH "COMMANDS"
.in +4
.CM	drive 
a: | b: | c: | d: | e: | f: | g: | h: | i: | j: <noargs>
.sp 1
changes default drive.  If you don't have such a drive, nothing happens.
.CM alias
alias <envstring>
.sp 1
assigns cmdstring to name.  name can now be used just as if it were
a built-in or external command.  cmdstring may contain history expressions
or variable substitutions. 
.sp 1
The syntax of this command is flexible - you can
specify alii (?) in the form 'name=subst','name subst','name = subst', or
'name =subst.'  However you need a space character
.bo
before
a single quote in order to specify an alias that contains blanks.
.sp 1
The
.bo
alias
and
.bo
set
commands are case sensitive, unlike their analogs in
.bo
COMMAND.COM.
.CM cat
cat	[<filelist>]
.sp 1
copies specified files to standard output.  If none are given, copies
standard input to standard output
.CM cp
cp | copy <filelist> <uname>
.sp 1
copies specified files to destination file or device.  If more than one
file is in the file list, <uname> must be a directory.
.CM cd
cd | chdir <dirname>
.sp 1
makes <dirname> the current default directory.
.CM chmod
chmod [-|+[arwhs]*] <filelist>
.sp 1
change file permissions for specified files
.sp 1
.nf
+r, -r turn on or off read permission - i.e. hide the file.
+w, -w turn on or off write permission.
+h, -h turn on or off hidden attribute - converse of r
+a, -a turn on or off archive attribute
+s, -s turns on or off the system attribute
.fi
.sp 1
Note that '-r' or '+rwh' are both valid syntax for switches. Also
new permission switches are permissable between file names with the
following warning:  I don't reset the masks between file names - if
you have a second batch of attribute changes on the command line, the
effect is additive.  If you're not careful, you could make a mess of
a files attributes.
.sp 1
If you don't specify any attribute switches, file attributes will be
set to 0, which means read,write,not hidden,not system, not 
modified since last backup.
.CM cls
cls <noargs>
.sp 1
clears the screen and homes the cursor.
.CM commands
commands <noargs>
.sp 1
prints a table of available built-in commands. (see above)
.CM del
del
.sp 1
synonym for rm.
.CM dir
dir
.sp 1
synonym for ls.
.CM	dirs
dirs <noargs>
.sp 1
lists the directories on the directory stack.  (see pushd and popd)
.CM du
du [drivename]
.sp 1
prints out remaining space on drive 
.bo
drivename.
If you leave off the drivename, it defaults to the current drive.
.CM dump
.sp 1
dump filespec [block [page]] | [segment:[offset]] [count]
.sp 1
.nf
Where a block is 64K bytes and a page is 256 bytes
Segment:offset are standard 8086 notation in hexadecimal
Count is the number of bytes to dump in decimal
.sp 1
This came from some anonymous public domain source, ported by me
.fi
.CM echo
echo <anything>
.sp 1
echos argument list to screen.
.bo
echo on
causes all commands to be echoed before execution (i.e. sets the verbose 
flag).
.bo
echo off
turns off the verbose flag.
.CM era
era
.sp 1
synonym for rm.
.CM error 
error <noargs>
.sp 1
prints returned value of last command to the screen.
.CM exit 
exit <noargs>
.sp 1
terminates execution of the currently running sub-shell.  If you are at
top level of execution, you will return to dos.
.CM fgrep 
fgrep <pattern> <filelist>
.sp 1
looks for unambiguous pattern <pattern> in <filelist>.
echos lines matching to the screen.
.CM history
history [size]
.sp 1
prints history list to standard output. If
.bo
size
is given, the number of commands history remembers is set to
.bo
size.
If you change the size, history 'forgets' all previous commands and resets its
counters to 0.
.CM ls 
ls | dir [-[alqctrR]] <filelist>
.sp 1
Lists files that match <filelist>
.sp 1
-a	all files, including system files are listed. '.' and '..' are
suppressed, but you know they're there if you need them, don't you?
.br
-l prints out file times, permissions, etc
.br
-q suppresses header line from display - useful when you want to pipe
stuff into another program.
.br
-c print as one column.
.br
-t sort by time, most recent last
.br
-R recurse through all encountered subdirectories.
.br
-r reverses sort order.
.CM md 
md | mkdir <uname>
.sp 1
make a directory. Prints an error if it can't be done
.CM mon
mon <noargs>
.sp 1
.bo
mon
prints to standard error a lot of cryptic information on system variables
thusly:
.sp 1
.in +4
.nf
Corg : 0000 Cend : 715a Dorg : 0002 Dend 167a
Uorg : 167a Uend : 327a mbot : 427c mtop 4800
sbot : 0000 PSP  : 490d STKSIZ : 00256 HEAPSIZ : 00064
dsval : 5033 csval : 491d
STKLOW : 0001 MEMRY : 4680
CS : 491d DS : 5033 SP : 4186
.fi
.in -4
.sp 1
You can probably figure out what most of the variables mean. If you
have source and have broken the shell, this command may help.
.CM more 
more [-[0-9]*] [<filelist>]
.sp 1
List file to screen with pauses
.sp 1
-n specify tab width when expanding tabs, where n is an integer.
more acts like 'cat' when redirected - you can concatenate files
in this manner.  If no files are specifed, standard input is 'mored.'
.CM mv 
mv [-v] <filelist> <uname>
.sp 1
moves specified file or files to target specifed by <uname>.
If there is more than one file in list, <uname> must be a directory
.sp 1
.bo
-v
will print out a message saying how it is accomplishing the move, which
is probably more interesting to me than you.
.CM popd 
popd <noargs>
.sp 1
returns to directory at top of directory stack.
.CM pushd 
pushd [<uname>]
.sp 1
save current working directory on directory stack, and changes
current working directory to <uname>.
.sp 1
If no <uname> is given, the current directory is swapped with the top
of the directory stack.
.CM pwd
pwd
.sp 1
prints current working directory to standard output.
.CM read
read vname0 [vname1 .. vnamen]
.sp 1
.bo
read
reads a line from standard input, and assigns each word from the line
to the corresponding environment variable specified by 
.bo
vname0 
through
.bo
vnamen.
.sp 1
EXAMPLE
.sp 1
.nf
.in +4
read a b	# read into environment variables a and b
.ti -4
you type
hello there
echo $a
.ti -4
the shell echos
hello
echo $b
.ti -4
there
.sp 1
.in -4
.br
.fi
If there are fewer environment variables specified on the command
line than there are words in the command line, the last environment
variable will contain the rest of the words on the line.
.sp 1
EXAMPLE
.sp 1
.nf
.in +4
read a b	# read into environment variables a and b
.ti -4
you type
hello there Mister Jones
echo $a
.ti -4
the shell echos
hello
echo $b
.ti -4
there Mister Jones
.sp 1
.in -4
.br
.fi
If there are more environment variables on the command line than words on
the line read from standard input, the left-over environment variables'
state will not change - if they were defined in the environment already,
their status won't change, and if they weren't defined, they stay
undefined.
.CM rd 
rd | rmdir <uname>
.sp 1
remove specified directory if possible.
.ne 6
.CM rm 
rm [-q] <filelist>
.sp 1
blows away all files in <filelist>. If -q is specified, will ask
if they should be removed.
.CM set 
set [<envstring>]
.sp 1
sets a string in the environment.  If you specify 'name=' with
no string after, it will remove it from the environment. If
you don't specify a string, set prints out current environment.
.sp 1
The syntax of this command is flexible - you can
specify set in the form 'set name=subst','set name subst','set name = subst', or
'set name =subst.'  However you need a space character
.bo
before
a single quote in order to specify a substitution string that contains blanks.
.CM sh
sh [ <arg1> .. <argn>] <scriptfile
.sp 1
forks a 'local' shell - i.e. saves all pertinent information about
the shell you're currently in and invokes the command processor function
recursively, with
.bo
scriptfile
as input.
The arguments are copied to $1 through $N environment strings,
overwriting the startup arguments.
.sp 1
.bo
sh
gives you a way to run scripts, without loading an extra copy of
shell.com or small.com.
.CM switchar
switchar [schar]
.sp 1
sets the DOS switch character to
.bo
schar.
If
.bo
schar
is not given, the current switch character is echoed.
.CM tee 
tee <uname>
.sp 1
Copies standard input to standard output, depositing a copy in
<uname>
.CM touch 
touch <filelist>
.sp 1
Makes the modification time of specified files the current date
and time.
.CM unalias
unalias aliasname
.sp 1
remove alias name from the alias list.
.br
.CM y  
y  <filelist>
.sp 1
copies standard input to standard output, and then copies the
specified files to standard output.  Sort of the opposite of
tee, in other words.
.sp 1
.in -4
.SH "Helpful hints"
.br
.sp 1
Use forward slashes in all path names. (See note below on switch characters) 
If you use DOS 3.0 or higher, this includes paths to transient programs.
.sp 1
put single quotes around arguments with semicolons in them, so they don't
turn into command delimiters.
.sp 1
The set command affects only the local shell's environment.  You
can 'exit' to command.com and the original environment is intact.
The local environment is 2K large - which is useful.
.sp 1
When using the Microsoft C compiler under pc-shell, the compiler has
a bad habit of look for parameters beginning with forward slashes in
the environment.  If you get their famous 'P0 : bad option' message,
try revising the environment.
.sp 1
Anything compiled with Microsoft C 3.0 that uses their system, exec, and
spawn functions will complain about forward slashes. Use the -b
option.
.sp 1
Sidekick has obnoxious ideas about when it should and shouldn't pop
up.  From the shell prompt you have to type <HOT-KEY><RETURN> to get
it to wake up.  The <RETURN> gets passed back to the shell.  Why?
Talk to Borland.  I use DOS function 3F (read from file or device) to
get keys, and you can't get any more vanilla than that.
.sp 1
.SH "Implementation notes"
.sp 1
DOS doesn't acknowledge a 'change default drive' command until
you issue a 'get current directory' call.  Why?  The only way
I figured this out is by disassembling command.com.
.sp 1
PC|MS-DOS has a limit of 20 file handles.  If you add a command that
opens files, make sure you catch the ctrl-break signal and close them.
Look at CAT.C or Y.C for examples.
.sp 1
DON'T REDIRECT INPUT INTO PRINT.  Print gets all hosed up.  Print has
lots of trouble in general with the pc-shell, and should be avoided.
.SH BUGS
.br
.sp 1
Due to the way that environment strings are expanded on the command line,
semicolons inside shell variables look like command seperators.  If you
enclose them in double quotes, the problem will go away.
.sp 1
I have noticed intermittent problems running on an AT, with
DOS 3.0, but have been unable to reproduce them on a PC.  I suspect bugs in
DOS 3.0 that are absent in 3.1. If you notice any consistent problems, send 
me a bug report.
.br
.SH HISTORY
.br
.in +4
.sp 1
.SH "V 2.0"
Minor bug fixes.  Started allocating command buffers out of heap rather
than stack, reducing stack usage.  Stopped releasing source with executables.
.SH "V 2.1"
Fixed the bug that made "mv foo .." throw away the source file and move it
to "..foo", (i.e. nowhere).  Added the sh command, that allows you to
run scripts without re-invoking the shell.
.SH "V 2.2"
Fixed the bug that locked pc up when a command line with a leading
.bo
;
was entered.  Changed fork code so that files with the extension .sh
are run as scripts. The equals sign in alias and set is now optional.
.br
.SH "V 2.3"
Removed information about versions 1.0 through 1.6 from documentation, adding
any relevant material to the tips or implementation notes sections.  
Added redirection of standard error, and checking for NOCLOBBER.  Implemented
execution of the prompt string.
.sp 1
.SH "V 2.4"
The problem running external programs written by IBM and Microsoft has finally
been traced to Aztecs fexecv function.  When is an ASCIIZ string not an ASCIIZ
string? When you need to put a single carriage return on the end of it.  THANKS
to Programmers Journal.  NOTE WELL that needing a carriage return in your
command tail for EXEC calls is note documented in the IBM PC-DOS technical
reference.
.sp 1
All references to the Aztec screen library have been removed, so this should
work on Rainbows, Compupros, AMPROs etc. etc. etc.  ls is slower, but its a
small price to pay.  If you want a fast version, one can be created for you if
you mail a contribution (hint,hint).
.sp 1
mv now will do the right thing when you try to move to a drive that has been
joined to a directory (DOS 3.1).  It also now has a verbose switch, which
tells you what exactly its doing.  When I started writing a mv, I had no idea
how many special cases I would have to take care of.
.SH "V 2.5"
Problem with using the wrong path to search for external programs solved.  Main
command line parsing changed so that NOGLOB could be used to suppress file
name expansion.
.SH "V 2.6"
Many stupid bugs in 2.5 are addressed (sorry folks).  I now peek inside
double quotes to do variable expansion.  Double quotes are now stripped.
.SH "V 2.7"
Added read.  Fixed bug in ls that would occasionally leave you in a
different directory. The exit command will now just pop you out a level of
shell execution, rather than kicking you out to the DOS prompt.
.SH "V 2.8"
Fixed bug in more that made it puke when you hit <CTRL>-<BREAK> at the
bottom of a screen-full.  Made move and cp preserve file times when they
have to create a target.
.SH "V 2.9"
Fixed bug in ls that caused it to get confused when you did an 'ls -R' on
a drive other than default. Also, stopped it from hanging on an empty
pipe.  Added the 'du' command - which is simply the last line of ls
-l.  Shell scripts will now be searched for on PATH.
.SH "V 3.0"
Bug fixed : when you switched to a disk for which there was no /tmp
directory, the shell couldn't open its pipe files.  Now pipe files
are opened in the default directory.  If you have a better idea, let
me know.
.sp 1
Added a critical error handler - if you get the abort, retry, ignore
message and abort, you return to the shell, rather than to DOS.
It is possible that all the memory allocated during the execution of
the offending command may not be freed, in which case the shell could
run out of memory.  Just give it the exit command and re-invoke if
that happens.
.sp 1
If you type something that the shell can't interpret as an internal
command, run as an external program or a script, I give command.com a
crack at it.  You can therefore now run dos batch files
transparently.  This slows things down somewhat if you type totally
bogus commands, so this feature can be turned off with "set NODOS=1".
.SH	"V 3.1"
Some general cleanup.  Added a function that makes an absolute path
from any relative path, which cleaned up mv, ls, and cp quite a bit.
.sp 1
Went back to .com format for pc-shell and small.  This makes the
archive file about 1K smaller on average.
.SH "V 3.2"
Added
.bo
-b
option in hopes of being more compatible with recalcitrant software
(like PC-NET) that chokes on forward slashes in paths.  It should be
noted that DOS 2.XX versions don't like forward slashes in the PATH
environment string.
.sp 1
Stopped closing standard handles 3 and 4 on startup.  This should
make weird things like Wordstar printer output being redirected to
the screen go away.
.sp 1
Single or double quotes around redirected file names are now permitted.
.SH "V 3.3"
Refined behavior of
.bo
-b
option.  Fixed aliasing in compound commands.  Deleted small.com from
distribution package because of lack of interest.  Fixed bug in add
to environment function, that made parts of the environment
disappear.  Made sidekick work slightly better.  Mv will now work on
directories under DOS V 3.0 or later.
.br
.SH "V 3.4"
Added logic to use the TEMP environment string to put pipe files
someplace useful.
.SH "V 3.5"
Tuning and linting.  Added pushd with no arguments.
.SH "LICENCING STATEMENT"
PC-SHELL is not in the public domain.  I, Kent Williams, retain all
rights of ownership over source and executables, unless I explicitly
transfer such rights to someone else.
.sp 1
All persons who come into possesion of PC-SHELL in the binary-only
version are entitled to use it however they wish.  No warranty is
given or implied.  All persons who come into possesion of PC-SHELL
are also entitled to redistribute the package in any way they see fit
providing that
.sp 1
.in +5
1) The package is distributed intact.  This means that the archive
file or distribution diskette contains PC-SHELL.COM, SMALL.COM and
SHELL.DOC, and that no changes have been made to any of these files.
.sp 1
2) That no charge is made, beyond a nominal fee for media duplication.
.br
.in -5
.sp 1
The above three paragraphs constitute an limited non-commercial
licence to PC-SHELL binaries to anyone who obtains it.  This
distiguishes it from being public domain only in that I retain
enough control over it to protect my own interests. (Take heart, hackers!)
.sp 1
All person who obtain copies of source code are granted unlimited
noncommercial use thereof.  This source code licence is not transferrable to
any other party, for any reason.
.sp 1
In addition, no modified versions of the program PC-SHELL may be
distributed by commercial or non-commercial means.
.sp 1
Any company wishing to purchase source licenses are subject to these
further restrictions.
.sp 1
.in +5
1) That no part of the source code is used, in any form in any
program sold commercially.
.sp 1
2) That a licence fee will be paid for each user within a company of
PC-SHELL, according to the rate schedule below.
.in -5
.SH "PRICING"
.nf
Noncommercial Personal Use - 25.00$
Commercial User	- 25.00$, plus 15.00$ per each additional user.
Commercial Distribution - by written agreement only
.br
.fi
.sp 1
For the above prices you will receive:
.in +5
.sp 1
1) Complete source code for shell, with makefile
.br
2) Source and executable for the text formatter used to prepare this
documentation.  NOTE: I didn't write this program, and am
distributing it as a service only.
.sp 1
3) Executable and documentation for NDMAKE, the best make available
for love or money on PC-DOS.  This is distributed as a service only.
Please honor the author's request for donations.
.sp 1
4) Source for as many other Unix Utilities as will fit on the disk.
Currently I am distributing cut, paste, make, nro, sed and tail.
I did not write these programs, and am distributing them as a
service only.
.sp 1
As always, miscellaneous contributions are welcome.  As anyone who
has contacted me is aware, I will provide as much help as I can, in
fixing bugs, providing updates, taking suggestions, etc., regardless
of monetary arrangements.
.sp 1
QUESTIONS COMMENTS BUGREPORTS GOTO
.br
.nf
KENT WILLIAMS
722 Rundell St.
Iowa City, IA 52240
(319) 338-6053 (HOME VOICE)

* UNIX is an unregistered trademark of AT&T.
